//--------------------------------------------------------------------------
//
// @function EqualizeHistDemo.cpp
// Display the source image.
// Display the RGB-histogram equalized image.
// Display the H-histogram equalized image, 
//			   S-histogram equalized image 
//         and V-histogram equalized image.
// 
//--------------------------------------------------------------------------


#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include <iostream>
//#include <stdio.h>

using namespace cv;
using namespace std;

int main( int argc, char** argv )
{
  	//--------------------------------------------------------------------------
	// Step 1. Initialization - prepare the source image img_BGR and display it
	//--------------------------------------------------------------------------
	// Load input_image, check for errors
	Mat input_image;

	if ( argc != 2 )
	{ 
		cout << "Usage: ./EqualizeHistDemo <input_image>" << endl;
		return -1; 
	}
	else 
	{	
		input_image = imread( argv[ 1 ], 1 );
		if( ! input_image.data )
		{
			cout << "Usage: ./EqualizeHistDemo <input_image>" << endl;
			return -1;
		}
	}

	// Scale-down the image to the size img_width x img_height.
	// (If we'll concatenate (horizontally) the three layers of the image 
	// into one large "merged" image, this image has 
	// to fit the screen of the notebook)
	int img_width = 400, img_height = 400;
	Mat img_BGR = Mat::zeros( img_height, img_width, input_image.type( ) );
	resize( input_image, img_BGR, img_BGR.size( ), 0, 0 ); // img_BGR will be our "source image"

	// Display the source image img_BGR
	namedWindow( "Source Image", CV_WINDOW_AUTOSIZE );
	imshow(      "Source Image", img_BGR );

	// -----------------------RGB case------------------------------------------

	//--------------------------------------------------------------------------
	// Step 2. Extract the R, G, B layers 
	//--------------------------------------------------------------------------
	vector<Mat> img_planes_BGR;
	split( img_BGR, img_planes_BGR );

	Mat img_R, img_G, img_B;
	img_planes_BGR[ 2 ].copyTo( img_R );
	img_planes_BGR[ 1 ].copyTo( img_G );
	img_planes_BGR[ 0 ].copyTo( img_B );

	//--------------------------------------------------------------------------
	// Step 3. For each layer (R, G, B) compute the histogram equalized layer
	//--------------------------------------------------------------------------
	Mat img_R_equalized, img_G_equalized, img_B_equalized;

	// Compute the histogram equalized image of the R layer
	equalizeHist( img_R, img_R_equalized );

	// Compute the histogram equalized image of the G layer
	equalizeHist( img_G, img_G_equalized );
	
	// Compute the histogram equalized image of the B layer
	equalizeHist( img_B, img_B_equalized );

	//--------------------------------------------------------------------------
	// Step 4. Merge the histogram equalized layers together, forming 
	// the RGB-histogram equalized image, 
	// and display the RGB-histogram equalized image
	//--------------------------------------------------------------------------
	vector<Mat> img_equalized_planes_BGR;
	img_equalized_planes_BGR.push_back( img_B_equalized );
	img_equalized_planes_BGR.push_back( img_G_equalized );
	img_equalized_planes_BGR.push_back( img_R_equalized );

	Mat img_histogram_equalized_BGR;
	merge( img_equalized_planes_BGR, img_histogram_equalized_BGR );

  	// Display the RGB-histogram equalized image
    namedWindow( "Histogram Equalization by R & G & B layers", CV_WINDOW_AUTOSIZE );
    imshow(      "Histogram Equalization by R & G & B layers", img_histogram_equalized_BGR );

	// -----------------------HSV case------------------------------------------

	//--------------------------------------------------------------------------
	// Step 5. Convert the source image img_BGR from BGR color space 
	// to HSV color space.
	//--------------------------------------------------------------------------
		
	// ...
	
	//--------------------------------------------------------------------------
	// Step 6. Extract the H, S, V layers 
	//--------------------------------------------------------------------------
	
	// ...

	//--------------------------------------------------------------------------
	// Step 7. For each layer (H, S, V) compute the histogram equalized layer
	//--------------------------------------------------------------------------
	
	// ...

	// Compute the histogram equalized image of the H layer
	// Mat img_H_scaled; // We want to store the original (not-scaled) img_H, also
	// double scale1 = (double) 255 / 179; // Scale the H layer 
										   // before histogram equalization
	// img_H_scaled = img_H * scale1;
	// equalizeHist( img_H_scaled, img_H_equalized );
	// double scale2 = (double) 179 / 255; // Scale the H layer 
										// after histogram equalization
	// img_H_equalized = img_H_equalized * scale2;

	// Compute the histogram equalized image of the S layer
	
	// ...
	
	// Compute the histogram equalized image of the V layer

	// ...

	//--------------------------------------------------------------------------
	// Step 8. Merge the histogram equalized layers together with the 
	// unchanged HSV layers, forming 
	// the H-histogram equalized image,
	// the S-histogram equalized image,
	// the V-histogram equalized image.
	//--------------------------------------------------------------------------

	// H-histogram equalized image
	
	// ...

	// S-histogram equalized image
	
	// ...

	// V-histogram equalized image
	
	// ...

	//--------------------------------------------------------------------------
	// Step 9. Take the H-histogram equalized image, 
	// S-histogram equalized image and 
	// V-histogram equalized image.
	// Convert them from HSV to BGR 
	// (since the display function "imshow" "wants" to get the BGR image)
	// and display them in one large "merged" image
	// (similarly to plotting together several Matlab Subplots in one Figure)
	//--------------------------------------------------------------------------

	// Convert the H-histogram equalized image from HSV to BGR
	
	// ...

	// Convert the S-histogram equalized image from HSV to BGR
	
	// ...

	// Convert the V-histogram equalized image from HSV to BGR
	
	// ...

	// Concatenate H, S, V-equalized images together into one large "merged" image
	// (similarly to plotting together several Matlab Subplots in one Figure)
	
	// ...

	// Display the H-histogram equalization, S-histogram equalization 
	// and V-histogram equalization in one image
	
	// ...

	//--------------------------------------------------------------------------
	// Step 10. Finalization
	//--------------------------------------------------------------------------
	waitKey( 0 ); // Wait for a pressed key

	return 0;
}
